/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef APERTURE_HH
#define APERTURE_HH
#include "DatEnv.hh"
#include "OperStateCondList.hh"
#include "TSeries.hh"
#include <vector>
#include <string>
#include <iosfwd>

namespace xsil {
   class MetaIO;
}

class channels {
public:
   channels(const std::string& chan, size_t N=32768);
   ~channels(void);
   void append(const TSeries& ts);
   bool empty(void) const;
   const std::string& name(void) const;
   const TSeries& ref_values(void) const;
   const TSeries& ref_rank(void) const;
   bool reject(void) const;
   void set_limits(size_t M=16);
   void set_reject(bool yorn);
   void test1D(const TSeries& ts, double CL);

private:
   size_t      _alloc;
   std::string _name;
   TSeries     _value;
   std::vector<double> _limits;
   TSeries     _rank;
   bool        _reject;
};

inline bool 
channels::empty(void) const {
   return _value.empty();
}

inline bool 
channels::reject(void) const {
   return _reject || empty();
}



class aperture : public DatEnv {
public:
   typedef Time::ulong_t gps_t;
public:
   aperture(int argc, const char* argv[]);
   ~aperture(void);
   void configure(const std::string& conf);
   void calculate(void);
   void ProcessData(void);
   void read_triggers(const std::string& file, gps_t start, gps_t stop);
   bool select_trigger(const xsil::MetaIO& mio) const;
   void veto_triggers(void);
private:

   //-----------------------------------   Trigger selection structure.
   struct trig_select {
      void critter(std::ostream& out) const;
      std::string field;
      std::string string;
      double      number;
      enum  {
	 s_pass,
	 s_string,
	 s_num_ge
      } type;
   };

   //-----------------------------------  Trigger veto structure
   struct trig_veto {
      std::string channel;
      double      val_min;
      double      val_max;
   };
   
private:
   std::string _config;
   double      _CL;
   double      _corr_limit;
   //-------------------------------------  OSC parameters
   std::string _oscfile;
   std::string _osc_cond;
   //-------------------------------------  Trigger file and selection
   std::string _trig_path;
   std::string _trig_table_name;
   std::string _trig_time_field;
   std::string _trig_time_ns_field;
   std::vector<trig_select> _trig_select_list;
   std::vector<trig_veto> _trig_veto_list;

   TSeries     _trig_series;
   size_t      _trig_total;
   typedef std::vector<channels> chan_vect;
   chan_vect   _channel;
   OperStateCondList _osc;
};

//======================================  inline methods
const std::string&
channels::name(void) const {
   return _name;
}

const TSeries&
channels::ref_values(void) const {
   return _value;
}

const TSeries&
channels::ref_rank(void) const {
   return _rank;
}

#endif // !defined(APERTURE_HH)
